package Lire::AsciiDlf::FilterExpr;

use strict;

our ($VERSION) = '$Revision: 1.11 $' =~ m!Revision: ([.\d]+)!;

# We need to use this module to see its subpackages
use Lire::FilterExpr;
use Lire::DataTypes qw/ :basic /;
use Carp;

sub is_numeric_value {
    my ( $self, $value ) = @_;

    if ( $self->_is_value_a_ref( $value) ) {
	my $name = $self->_refname_from_value( $value );
	if ( $self->{container}->has_param( $name ) ) {
	    return is_numeric_type( $self->{container}->param( $name )->type );
	} else {
	    return is_numeric_type( $self->{container}->schema()->field( $name )->type );
	}
    } else {
	return check_number( $value );
    }
}

sub make_id_value {
    my ( $self, $value ) = @_;

    if ( $self->_is_value_a_ref( $value ) ) {
	my $name = $self->_refname_from_value( $value );
	if ( $self->{container}->has_param( $name ) ) {
	    my $v = $self->{container}->param( $name )->value();
	    return $v;
	} else {
	    my $pos = $self->{container}->schema()->field( $name )->pos();
	    return "dlf[$pos]";
	}
    } else {
	return $value;
    }
}

sub make_eval_value {
    my ( $self, $value ) = @_;

    my $is_num = $self->is_numeric_value( $value );
    if ( $self->_is_value_a_ref( $value ) ) {
	my $name = $self->_refname_from_value( $value );
	if ( $self->{container}->has_param( $name ) ) {
	    my $v = $self->{container}->param( $name )->value();
            if ( defined $v ) {
                return $is_num ? $v : qq{q{$v}};
            } else {
                return 'undef';
            }
	} else {
	    my $pos = $self->{container}->schema()->field( $name )->pos();
	    return "\$_[0][$pos]";
	}
    } else {
        if (defined $value) {
            return $is_num ? $value : qq{q{$value}};
        } else {
            return 'undef';
        }
    }
}

sub compile {
    croak "Unimplemented: ", __PACKAGE__, "::compile";
}

package Lire::AsciiDlf::FilterExpr::BinaryExpr;

use base qw/ Lire::AsciiDlf::FilterExpr /;

use Carp;

sub id {
    my ($self) = @_;
    my $arg1 = $self->make_id_value( $self->arg1 );
    my $arg2 = $self->make_id_value( $self->arg2 );
    my $op   = $self->make_eval_op();

    return "$op($arg1 $arg2)";
}

sub compile {
    my ($self) = @_;
    my $arg1 = $self->make_eval_value( $self->arg1 );
    my $arg2 = $self->make_eval_value( $self->arg2 );
    my $op   = $self->make_eval_op();

    my $code = qq! sub { defined $arg1 && defined $arg2 && $arg1 $op $arg2 } !;
    my $sub  = eval $code;
    croak "error compiling expression ($code): $@" if $@;

    return $sub;
}

sub make_eval_op {
    croak "Unimplemented: ", __PACKAGE__, "::make_eval_op";
}

package Lire::AsciiDlf::FilterExpr::Eq;

use base qw/ Lire::AsciiDlf::FilterExpr::BinaryExpr Lire::FilterExpr::Eq /;

sub make_eval_op {
    my ( $self ) = @_;

    if ( $self->is_numeric_type( $self->arg1) &&
	 $self->is_numeric_type( $self->arg2) ) {
	return "==";
    } else {
	return "eq";
    }
}

package Lire::AsciiDlf::FilterExpr::Ne;

use base qw/ Lire::AsciiDlf::FilterExpr::BinaryExpr Lire::FilterExpr::Ne /;

sub make_eval_op {
    my ( $self ) = @_;

    if ( $self->is_numeric_type( $self->arg1) &&
	 $self->is_numeric_type( $self->arg2) ) {
	return "!=";
    } else {
	return "ne";
    }
}

package Lire::AsciiDlf::FilterExpr::Lt;

use base qw/ Lire::AsciiDlf::FilterExpr::BinaryExpr Lire::FilterExpr::Lt /;

sub make_eval_op {
    return "<";
}

package Lire::AsciiDlf::FilterExpr::Le;

use base qw/ Lire::AsciiDlf::FilterExpr::BinaryExpr Lire::FilterExpr::Le /;

sub make_eval_op {
    return "<=";
}

package Lire::AsciiDlf::FilterExpr::Gt;

use base qw/ Lire::AsciiDlf::FilterExpr::BinaryExpr Lire::FilterExpr::Gt /;

sub make_eval_op {
    return ">";
}

package Lire::AsciiDlf::FilterExpr::Ge;

use base qw/ Lire::AsciiDlf::FilterExpr::BinaryExpr Lire::FilterExpr::Ge /;

sub make_eval_op {
    return ">=";
}

package Lire::AsciiDlf::FilterExpr::Match;

use Carp;

use base qw/ Lire::AsciiDlf::FilterExpr Lire::FilterExpr::Match /;

sub id {
    my ( $self ) = @_;

    my $val = $self->make_id_value( $self->value );
    my $re  = "/" . $self->make_id_value( $self->re ) . "/";
    $re .= "i" unless $self->case_sensitive;

    return "match($val $re)";
}

sub compile {
    my ( $self ) = @_;

    my $val = $self->make_eval_value( $self->value );
    my $re = $self->re;
    my $flags = "";
    $flags .= "i" unless $self->case_sensitive;
    if ( $self->_is_value_a_ref( $re ) ) {
	my $name = $self->_refname_from_value( $re );
        $re = $self->{container}->param( $name )->value();
    }

    my $code = qq|sub { defined $val && $val =~ m<$re>$flags }|;
    my $sub  = eval $code;
    croak "error compiling expression ($code): $@" if $@;

    return $sub;
}

package Lire::AsciiDlf::FilterExpr::Value;

use base qw/ Lire::AsciiDlf::FilterExpr Lire::FilterExpr::Value /;

sub id {
    my ( $self ) = @_;

    my $val = $self->make_eval_value( $self->value );

    return "value($val)";
}

sub compile {
    my ( $self ) = @_;

    my $val = $self->make_eval_value( $self->value );

    return sub { $val ? 1 : 0 };
}

package Lire::AsciiDlf::FilterExpr::Not;

use base qw/ Lire::AsciiDlf::FilterExpr Lire::FilterExpr::Not /;

sub id {
    my ($self) = @_;

    my $id = $self->expr->id;
    return "not( $id )";
}

sub compile {
    my ($self) = @_;

    my $sub = $self->expr->compile;
    return sub { ! $sub->( $_[0] ) };
}

package Lire::AsciiDlf::FilterExpr::And;

use base qw/ Lire::AsciiDlf::FilterExpr Lire::FilterExpr::And /;

sub id {
    my ($self) = @_;

    my @ids = ();
    foreach my $e ( @{$self->expr()} ) {
	push @ids, $e->id;
    }

    return "and( " . join( ", ", @ids) . " )";
}

sub compile {
    my ($self) = @_;

    my @expr = ();
    foreach my $e ( @{$self->expr()} ) {
	push @expr, $e->compile;
    }

    return sub {
	foreach my $e ( @expr ) {
	    return 0 unless $e->( $_[0] );
	}
	return 1;
    };
}

package Lire::AsciiDlf::FilterExpr::Or;

use base qw/ Lire::AsciiDlf::FilterExpr Lire::FilterExpr::Or /;

sub id {
    my ($self) = @_;

    my @ids = ();
    foreach my $e ( @{$self->expr()} ) {
	push @ids, $e->id;
    }

    return "or( " . join( ", ", @ids) . " )";
}

sub compile {
    my ($self) = @_;

    my @expr = ();
    foreach my $e ( @{$self->expr()} ) {
	push @expr, $e->compile;
    }

    return sub {
	foreach my $e ( @expr ) {
	    return 1 if $e->( $_[0] );
	}
	return 0;
    };
}

# keep perl happy
1;

__END__

=pod

=head1 NAME

Lire::FilterExpr -

=head1 SYNOPSIS


=head1 DESCRIPTION

=head1 VERSION

$Id: FilterExpr.pm,v 1.11 2003/02/03 00:28:03 flacoste Exp $

=head1 COPYRIGHT

Copyright (C) 2001 Stichting LogReport Foundation LogReport@LogReport.org

This file is part of Lire.

Lire is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see COPYING); if not, check with
http://www.gnu.org/copyleft/gpl.html or write to the Free Software 
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

=head1 AUTHOR

Francis J. Lacoste <flacoste@logreport.org>

=cut
