package Lire::Config;

use strict;

use vars qw/ $VERSION $PACKAGE $prefix $sysconfdir $exec_prefix $pkgdatadir
             $datadir $libexecdir $xsldir $dsssldir
             $gzip $sendmail $tar $ploticus $ghostscript
             $LR_DEBUG $LR_KEEP $LR_SCALE_BYTES $LR_SCALE_NUMBER $LR_SCALE_SEC
             $LR_ARCHIVE $LR_IMAGE_FORMAT $LR_IMAGE_STYLE $LR_IMAGE_RESOLUTION
             $LR_WEEK_NUMBERING $LR_SQL_MAX_SIZE
             $dbi_available
             $SINGLETON
            /;

use Lire::Logger;
use Lire::Config::Build qw(ac_info ac_path);

use Carp;

=pod

=head1 NAME

Lire::Config - import configure variables to perl

=head1 SYNOPSIS

 use Lire::Config;

 Lire::Config->init();

 my $cfg->value = Lire::Config->get( 'lr_schemas_path' );

 my $var = Lire::Config->get_var( 'lr_schemas_path' );

=head1 DESCRIPTION

This package provides the API to access the Lire configuration.

=head2 METHODS

These methods can be called directly on a Lire::Config instance or the
Lire::Config package itself. In the last case, it is the same thing
than caling the method on the object returned by the instance()
method.

=cut

eval "use DBI";
$dbi_available = 1 unless $@;

$SINGLETON = new Lire::Config::XMLFilesConfig();

sub lire_program_config_init {
    $SINGLETON->init();
    init_vars();
}

sub init_vars {
    # VERSION does not hold this module's version, but the Lire version.
    # ($VERSION)        = '$Release:$' =~ m!Release: ([.\d]+)!;
    $VERSION        = ac_info('VERSION');

    $PACKAGE        = ac_info('PACKAGE');
    $prefix         = ac_info('prefix');
    $sysconfdir     = ac_info('sysconfdir');
    $exec_prefix    = ac_info('exec_prefix');
    $datadir        = ac_info('datadir');
    $pkgdatadir     = ac_path('datadir', 'PACKAGE');
    $libexecdir     = ac_info('libexecdir');

    # Transform potential warning in debug
    # messages. Since this variabels aren't accessed to
    # be used.
    local $SIG{__WARN__} = sub { lr_debug( @_ ) };
    $gzip           = $SINGLETON->get( 'gzip_path' );
    $sendmail       = $SINGLETON->get( 'sendmail_path' );
    $tar            = $SINGLETON->get( 'tar_path' );
    $ploticus       = $SINGLETON->get( 'ploticus_path' );
    $ghostscript    = $SINGLETON->get( 'ghostscript_path' );

    # We turn undef into 0 for backward compatibility
    $LR_DEBUG       = $SINGLETON->get( 'lr_debug' ) || 0;
    $LR_KEEP        = $SINGLETON->get( 'lr_keep' ) || 0;
    $LR_ARCHIVE     = $SINGLETON->get( 'lr_archive' ) || 0;
    $LR_SCALE_BYTES = $SINGLETON->get( 'lr_scale_bytes' ) || 0;
    $LR_SCALE_NUMBER    = $SINGLETON->get( 'lr_scale_numbers' ) || 0;
    $LR_SCALE_SEC       = $SINGLETON->get( 'lr_scale_seconds' ) || 0;
    $LR_WEEK_NUMBERING  = $SINGLETON->get( 'lr_week_numbering' );
    $LR_IMAGE_FORMAT    = $SINGLETON->get( 'lr_image_format' );
    $LR_IMAGE_STYLE     = $SINGLETON->get( 'lr_image_style' );
    $LR_IMAGE_RESOLUTION = $SINGLETON->get( 'lr_image_resolution' );
    $LR_SQL_MAX_SIZE    = $SINGLETON->get( 'lr_sql_max_size' );
}

sub connect_dbi {
    die "DBI isn't available\n" unless $dbi_available;

    my $dbi_uri  = Lire::Config->get( 'lr_dbi_uri' );
    my $dbi_user = Lire::Config->get( 'lr_dbi_user' );
    my $passwd_file = Lire::Config->get( 'lr_dbi_passwd_file' );

    die "lr_dbi_uri isn't set\n"
      unless defined $dbi_uri;

    die "only MySQL database is currently supported\n"
      unless index( $dbi_uri, 'DBI::mysql' ) == 0;

    die "lr_dbi_user isn't set\n"
      unless defined $dbi_user;

    die "lr_dbi_passwd_file isn't set\n"
      unless defined $passwd_file;

    open PASSWD, $passwd_file
      or die "can't open $passwd_file: $!\n";

    my $passwd = <PASSWD>;
    chomp $passwd;
    close PASSWD;

    DBI->connect( $dbi_uri, $dbi_user, $passwd )
      or die "can't connect to DBI $dbi_uri: ", $DBI::errstr, "\n";
}


=pod

=head2 instance()

Returns the singleton Lire::Config object from which configuration can
be queried.

=cut

sub instance {
    return $SINGLETON;
}

=pod

=head2 init()

This method loads parses the configuration files. It should be called
prior to using the get() method to obtain configuration data. This
method will throw an exception in the case there is an invalid
parameter in the configuration.

=head2 get( $varname )

Returns the configuration value for the $varname configuration
variable. init() should have been called to load the configuration
data before using this method. It will croak() otherwise. If the
$varname is unknown, an exception will also be thrown.

=head2 get_var( $varname )

Returns the configuration value for the $varname configuration
variable as the configuration object. init() should have been called
to load the configuration data before using this method. It will
croak() otherwise. If the $varname is unknown, an exception will also
be thrown.

=head2 config_spec()

Returns the configuration specification used by the current
configuration.

=head2 config_spec_path()

Returns a list of directories which will be searched for configuration
specification file. By default, configuration specifications are only
looked for in F<<datadir>/lire/config-spec>.

=head2 add_config_spec_path_dir( $dir )

This method adds a directory to list of directories that will is used
to search for configuration specification files.

=head2 del_config_spec_path_dir( $dir )

This method removes a directory from the list of directories that is
used to search for configuration specification files.

=head2 config_files()

Returns the list of configuration files that will be used for this
configuration.

=head2 add_config_path( $path )

Adds configuration files to the list of files that are part of the
configuration. Directories will be scanned recursively.

=head2 new_config_file( $file )

Creates a new, empty configuration file and adds it to the list of
configuration files.

=head2 add_config_file( $file )

Adds a configuration file to the list of files that will be parsed to
initialize the configuration.

=head2 del_config_file( $file )

Removes a configuration file from the list of file that are part of
the configuration..

=head2 get_config_file($file)

Get a parsed configuration file object from the list of files.

=cut

BEGIN {
    no strict 'refs';

    # Autogenerate methods which delegates to the singleton.
    foreach my $method ( qw/init config_spec config_spec_path 
                            add_config_spec_path_dir del_config_spec_path_dir
                            add_config_path
                            add_config_file del_config_file config_files
                            get_config_file new_config_file
                            get get_var/ )
    {
        *{$method} = sub { my $pkg = shift; $SINGLETON->$method( @_ ) };
    }
}

package Lire::Config::XMLFilesConfig;

use base qw/Lire::Config/;

use Lire::Config::Parser;
use Lire::Config::SpecParser;
use Lire::Utils 'tilde_expand';
use Lire::Config::Build 'ac_path';

use Carp;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $self = bless {
        _config_spec_dirs => [ ac_path('datadir', 'PACKAGE') . "/config-spec" ],
        _config_files => [],
        _parsed_config => {},
        _spec => undef,
    }, $class;

    foreach(( ac_path('datadir', 'PACKAGE') ."/defaults",
                  ac_path('sysconfdir', 'PACKAGE') . "/config",
                  tilde_expand('~/.lire/config')))
        {
            $self->add_config_path($_) if -d $_;
    }

    return $self;
}

sub config_spec {
    my $self = $_[0];

   croak( "config_spec() called while configuration wasn't initialized" )
     unless $self->{_spec};

    return $self->{_spec};
}

sub config_spec_path {
    my $self = $_[0];

    @{$self->{_config_spec_dirs}};
}

sub add_config_spec_path_dir {
    my ($self, $dir) = @_;

    croak "missing 'dir' argument"
      unless $dir;

    unshift @{$self->{_config_spec_dirs}}, $dir;
    delete $self->{_spec};

    return;
}

sub del_config_spec_path_dir {
    my ($self, $dir) = @_;

    croak "missing 'dir' argument"
      unless $dir;

    @{$self->{_config_spec_dirs}} =
      grep { $_ ne $dir } $self->config_spec_path;

    delete $self->{_spec};

    return;
}

sub config_files {
    my ($self, $dir) = @_;

    return @{$self->{_config_files}}
}

sub add_config_path {
    my ($self, $path) = @_;

    croak "missing 'path' argument"
      unless $path;

    croak "path '$path' doesn't exists"
      unless -e $path;

    croak "path '$path' isn't readable"
      unless -r $path;

    if(-d $path) {
        opendir(DIR, $path)
          or croak("Can't open directory '$path': $!");
        foreach my $ent (readdir(DIR)) {
            next unless $ent =~ /\.xml$/;
            next if $ent =~ /^[.#]/;
            $self->add_config_path("$path/$ent");
        }
        close(DIR);
    } else {
        $self->add_config_file($path);
    }

    return;
}


sub new_config_file {
    my ($self, $file) = @_;

    croak "missing 'file' argument"
      unless $file;

    croak "file '$file' already exists"
      if -e $file;

    open(FH, "> $file")
        or croak("Can't open $file: $!");

    print FH "<!-- Created by lr_interactive -->\n";
    print FH '<config xmlns="http://www.logreport.org/LRCML/">', "\n";
    print FH "<global></global>\n";
    print FH "</config>\n";

    close(FH)
        or croak("Can't close $file: $!");

    return $self->add_config_file($file);
}

sub add_config_file {
    my ($self, $file) = @_;

    croak "missing 'file' argument"
      unless $file;

    croak "file '$file' doesn't exists"
      unless -e $file;

    croak "file '$file' is not readable"
      unless -r $file;

    push @{$self->{_config_files}}, $file;
    $self->_load_config_file($file) if $self->{_spec};

    return;
}

sub del_config_file {
    my ($self, $file) = @_;

    croak "missing 'file' argument"
      unless $file;

    @{$self->{_config_files}} = grep { $_ ne $file } $self->config_files;
    delete $self->{_parsed_config}{$file};

    return;
}

sub get_config_file {
    my ($self, $file) = @_;

    croak "missing 'file' argument"
      unless $file;

    croak("file '$file' is not a known configuration file")
      unless exists $self->{_parsed_config}{$file};

    return $self->{_parsed_config}{$file};
}

sub init {
    my $self = $_[0];

    return if $self->{_spec};

    $self->set_job_id($ENV{LR_JOB_ID});

    $self->_load_spec;
    $self->_load_config_files;
}

sub set_job_id {
    my ($self, $job_id) = @_;

    if(defined $job_id && $job_id ne '') {
        $self->{job_id} = $job_id;
    } else {
        undef $self->{job_id};
    }
}

sub get {
    return shift->get_var( @_ )->as_value;
}

sub get_var {
   my ($self, $name) = @_;

   croak("Configuration queried without initialization")
     unless $self->{_spec};

   croak "No such configuration variable: $name"
      unless $self->{_spec}->has_component($name);

    if(defined $self->{job_id}) {
        my $jobid = $self->{job_id};
        foreach my $file (reverse $self->config_files) {
            my $cfg = $self->{_parsed_config}{$file}->job($jobid);
            next unless defined $cfg;
            next unless $cfg->is_set($name);
            return $cfg->get($name);
        }
    }

   foreach my $file (reverse $self->config_files) {
       my $cfg = $self->{_parsed_config}{$file}->global;
       return $cfg->get($name)
         if $cfg->is_set($name)
   }

   return $self->{_spec}->get( $name )->instance;
}

sub _load_spec {
    my $self = $_[0];

    my $parser = new Lire::Config::SpecParser;
    foreach my $dir ( $self->config_spec_path ) {
        $parser->merge_specifications_dir( $dir );
    }
    $self->{_spec} = $parser->configspec;
}

sub _load_config_files {
    my $self = $_[0];

    $self->{_parsed_config} = {};
    foreach my $file ( $self->config_files ) {
        $self->_load_config_file( $file );
    }
}

sub _load_config_file {
    my ($self, $file) = @_;

    my $parser = new Lire::Config::Parser( spec => $self->{_spec} );
    $self->{_parsed_config}{$file} = $parser->load_config_file( $file );
}

# keep perl happy
1;

__END__

=pod

=head1 SEE ALSO

Lire::Config::Spec(3pm) Lire::Config::Values(3pm)

=head1 VERSION

$Id: Config.pm,v 1.22 2003/05/04 15:34:05 flacoste Exp $

=head1 AUTHORS

Wessel Dankers <wsl@logreport.org>
Francis J. Lacoste <flacoste@logreport.org>
Joost van Baal <joostvb@logreport.org>

=head1 COPYRIGHT

Copyright (C) 2001-2003 Stichting LogReport Foundation LogReport@LogReport.org

This file is part of Lire.

Lire is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see COPYING); if not, check with
http://www.gnu.org/copyleft/gpl.html or write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

=cut
