package tests::DlfConverterProcessTest;

use strict;

use base qw/ Lire::Test::DlfConverterTestCase /;
our ($VERSION) = '$Revision: 1.4 $' =~ m!Revision: ([.\d]+)!;

use Lire::DlfConverterProcess;

my %schemas =
(
 test1 => <<EOF,
<lire:dlf-schema superservice="test1" timestamp="time"
 xmlns:lire="http://www.logreport.org/LDSML/">
<lire:field name="time" type="timestamp"/>
<lire:field name="code" type="string"/>
<lire:field name="message" type="string"/>
</lire:dlf-schema>
EOF
 test2 => <<EOF,
<lire:dlf-schema superservice="test2" timestamp="time_start"
 xmlns:lire="http://www.logreport.org/LDSML/">
<lire:field name="time_start" type="timestamp"/>
<lire:field name="file" type="filename"/>
<lire:field name="result" type="string"/>
</lire:dlf-schema>
EOF
);

my %logs =
  (
   test_line =>
   {
    log => <<EOF,
schema=test1;time=90000;code=500;message=This is a long message
schema=test1;
schema=test2;time_start=1023456;file=/var/tmp;result=OK
ignore=1;key=another line to be ignored
schema=test1;time=
schema=test1;time=100000;message=Another message
ignore=1;key=this line will be ignored
EOF
    error_count => 1,
    line_count => 7,
    ignored_count => 2,
    saved_count => 0,
    dlf_count => 4,
    dlf =>
    {
     test1 =>
     [
      {
       time    => 90000,
       code    => 500,
       message => "This is a long message",
      },
      {
       time    => undef,
       code    => undef,
       message => undef,
      },
      {
       time    => 100000,
       code    => undef,
       message => "Another message",
      },
     ],
     test2 =>
     [
      {
       time_start => 1023456,
       file => "/var/tmp",
       result => "OK",
      },
     ],
    },
   },

   continuation =>
   {
    log => <<EOF,
schema=test1;time=90000;code=500;message=This is a long message;save=1
schema=test2;time_start=1023456;file=/var/tmp;result=OK;save=1
EOF
    error_count => 0,
    line_count => 2,
    ignored_count => 0,
    saved_count => 2,
    dlf_count => 0,
    dlf => {},
   },
  );

# Modify for file test
$logs{test_file} = { %{$logs{test_line}} };
$logs{test_file}{name} = "test_file";
$logs{test_file}{line_count} = 0;
$logs{test_file}{ignored_count} = undef;
$logs{test_file}{saved_count} = undef;

# Continuation results
my $cont_results =
  {
    log => "",
    error_count => 0,
    line_count => 4, # 2 saved + 2 of original log
    ignored_count => 0,
    saved_count => 2, # 2 of the original log
    dlf_count => 2, # 2 saved
    dlf =>
    {
     test1 => [
               {
                time    => 90000,
                code    => 500,
                message => "This is a long message",
               },
              ],
     test2 => [
               {
                time_start => 1023456,
                file => "/var/tmp",
                result => "OK",
               },
              ],
    },
  };

sub schema_fixtures {
    \%schemas;
}

sub log_source_fixtures {
    my $r = { map { $_ => $logs{$_}{log} } keys %logs };

    return $r;
}

my @converters =
  (
   new tests::DlfConverterProcessTest::DlfConverter( "line", 1 ),
   new tests::DlfConverterProcessTest::DlfConverter( "file", 0 )
  );

sub converter_fixtures {
    \@converters;
}

sub test_new {
    my $self = $_[0];

    my $p = new Lire::DlfConverterProcess( $self->log_source( "test_line"),
                                           $self->dlf_store );

    $self->assert( UNIVERSAL::isa( $p, "Lire::DlfConverterProcess" ),
                   "new() didn't returned proper instance" );
    $self->assert_equals( $self->log_source( "test_line" ), $p->log_source );
    $self->assert_equals( $self->dlf_store, $p->dlf_store );
}

sub test_convert_source_line {
    my $self = $_[0];

    $self->log_source( "test_line" )->converter( "line" );
    my $p = new Lire::DlfConverterProcess( $self->log_source( "test_line" ),
                                           $self->dlf_store );
    $self->assert_not_null( $p, "new() returned undef" );

    $p->convert_source;

    $self->assert_dlf_converter_match_results( $logs{test_line}, $p );

    eval { $p->convert_source };
    $self->assert_not_null( $@, "calling convert_source twice should fail" );
}

sub test_convert_source_file {
    my $self = $_[0];

    $self->log_source( "test_file" )->converter( "file" );
    my $p = new Lire::DlfConverterProcess( $self->log_source( "test_file" ),
                                           $self->dlf_store );
    $self->assert_not_null( $p, "new() returned undef" );

    $p->convert_source;
    $self->assert_dlf_converter_match_results( $logs{test_file}, $p );
}

sub test_convert_continuation {
    my $self = $_[0];

    $self->log_source( "continuation" )->converter( "line" );
    my $p = new Lire::DlfConverterProcess( $self->log_source( "continuation" ),
                                           $self->dlf_store );
    $self->assert_not_null( $p, "new() returned undef" );

    $p->convert_source;
    $self->assert_dlf_converter_match_results( $logs{continuation}, $p );

    # Should process saved lines now
    $p = new Lire::DlfConverterProcess( $self->log_source( "continuation" ),
                                        $self->dlf_store );
    $p->convert_source;
    $self->assert_dlf_converter_match_results( $cont_results, $p );
}

package tests::DlfConverterProcessTest::DlfConverter;

use base qw/Lire::DlfConverter/;

sub new {
    bless { name => $_[1],
            handle_lines => $_[2],
          }, $_[0];
}

sub name { $_[0]{name} };

sub handle_log_lines { $_[0]{handle_lines} }

sub schemas { return ("test1", "test2") };

sub init_dlf_converter {}

sub process_log_file {
    my ( $self, $process, $fh ) = @_;
    while ( <$fh> ) {
        chomp $_;
        $self->process_log_line( $process, $_ );
    }
}

sub process_log_line {
    my ( $self, $process, $line ) = @_;

    my %r = ();
    my @fields = split /;/, $line;
    foreach my $f ( @fields ) {
        my ( $key, $value ) = $f =~ /^(\w+)=(.+)$/;

        if ( defined $key && defined $value ) {
            $r{$key} = $value;
        } else {
            $process->error( "contains an invalid field", $line );
            return;
        }
    }

    if ( $r{ignore}) {
        $process->ignore_log_line( $line )
          if $self->{handle_lines};
    } elsif ( $r{save}) {
        $line =~ s/save=1/save=0/;
        $process->save_log_line( $line )
          if $self->{handle_lines};
    } else {
        $process->write_dlf( $r{schema}, \%r );
    }
}

sub finish_conversion {}


1;
