package tests::DlfSchemaTest;

use strict;

use base qw/ Lire::Test::TestCase /;

use Lire::DlfSchema;
use File::Temp qw/:mktemp tempdir /;
use Lire::Test::HashConfig;
use Symbol;
use Carp;

my $test_schema = <<EOF;
<lire:dlf-schema superservice="test" timestamp="time_start"
 xmlns:lire="http://www.logreport.org/LDSML/">
<lire:field name="time_start" type="timestamp"/>
<lire:field name="field_1" type="string"/>
<lire:field name="field_2" type="string"/>
<lire:field name="int_3" type="int"/>
<lire:field name="url_4" type="url"/>
</lire:dlf-schema>
EOF

sub new {
    my $self = shift()->SUPER::new( @_ );

    $self->{tmpdir} = tempdir( __PACKAGE__ . "XXXXXX",
                               TMPDIR => 1, CLEANUP => 1 );

    $self->{schemapath} = $self->{tmpdir} . "/test.xml";
    mkdir $self->{tmpdir} . "/schemas", 0755
      or croak "failed to make directory: $self->{tmpdir}/schemas";
    $self->create_schema( $test_schema, "test", $self->{schemapath} );
    $self->create_schema( $test_schema, "test-exttest",
                          "$self->{tmpdir}/schemas/test-exttest.xml" );

    return $self;
}

sub create_schema {
    my ( $self, $template, $name, $path ) = @_;

    $template =~ s/superservice="test"/superservice="$name"/;
    my $fh = gensym;
    open $fh, "> $path"
      or die "can't create $name schema: $path: $!";
    print $fh $template;
    close $fh;
}

sub set_up {
    my $self = shift->SUPER::set_up();

    $self->{cfg}{lr_schemas_path} = [ $self->{tmpdir},
                                      $self->{tmpdir} . "/schemas" ];
}

sub test_schema_load {
    my $self = $_[0];

    # To make sure that the change to the schema file
    # is noticed
    my $or_schema = Lire::DlfSchema::load_schema( "test" );
    $self->assert_not_null( $or_schema, "load_schema() returned undef" );

    # Should return cached copy
    my $schema = Lire::DlfSchema::load_schema( "test" );
    $self->assert_equals( $or_schema, $schema );

    # Modify mtime to force a reload
    my $now = time + 5;
    utime $now, $now, $self->{schemapath};

    $schema = Lire::DlfSchema::load_schema( "test" );
    $self->assert_not_equals( $self->{schema}, $schema );
}

sub test_has_schema {
    my $self = $_[0];

    $self->assert( Lire::DlfSchema->has_schema( "test"),
                   "has_schema( 'test' ) failed" );
    $self->assert( Lire::DlfSchema->has_schema( "test-exttest"),
                   "has_schema( 'test-exttest' ) failed" );
    $self->assert( ! Lire::DlfSchema->has_schema( "test2"),
                   "has_schema( 'test2' ) failed" );

    $self->assert_died( sub {Lire::DlfSchema->has_schema()},
                        qr/missing 'schema_name' parameter/ );

    $self->assert_died( sub {Lire::DlfSchema->has_schema( "../test" ) },
                        qr/invalid schema name/ );
}

sub test_has_superservice {
    my $self = $_[0];

    $self->assert( Lire::DlfSchema->has_superservice( "test"),
                   "has_schema( 'test' ) failed" );

    $self->assert( ! Lire::DlfSchema->has_superservice( "test2"),
                   "has_schema( 'test2' ) should have failed" );

    $self->assert_died( sub {Lire::DlfSchema->has_superservice()},
                        qr/missing 'superservice' parameter/ );


    $self->assert_died( sub {Lire::DlfSchema->has_superservice( "test-exttest" ) },
                        qr/invalid superservice schema name/ );

    $self->assert_died( sub {Lire::DlfSchema->has_superservice( "../test" ) },
                        qr/invalid superservice schema name/ );


}

sub test_superservices {
    my $self = $_[0];

    my @super = Lire::DlfSchema->superservices();
    $self->assert_deep_equals( [ "test" ], \@super );
}

sub test_schemas {
    my $self = $_[0];

    my @schemas = Lire::DlfSchema->schemas();
    $self->assert_deep_equals( [ "test", "test-exttest" ], \@schemas );
}

1;



