package tests::DlfStoreTest;

use strict;

use base qw/ Lire::Test::TestCase /;

use Lire::DlfStore;
use File::Temp qw/tempdir/;
use File::Basename qw/dirname/;
use Lire::Utils qw/create_file/;
use Cwd qw/realpath/;

sub new {
    my $self = shift()->SUPER::new( @_ );

    $self->{workingdir} = tempdir( __PACKAGE__ . "XXXXXX",
                                   CLEANUP => 1, TMPDIR => 1,
                                 );
    $self->{schemasdir} = realpath( dirname(__FILE__) . "/schemas" );

    $self;
}

sub set_up {
    my $self = shift->SUPER::set_up();

    $self->{cfg}{lr_schemas_path} = [ $self->{schemasdir} ];
}

sub test_open_create {
    my $self = $_[0];

    my $path  = $self->{workingdir} . "/test_open_create";
    my $store = Lire::DlfStore->open( $path, 1 );
    $self->assert_not_null( $store, "Lire::DlfStore->open returned undef" );

    $self->assert_equals( $path, $store->path );

    $self->assert( -d $path, "open() didn't create the directory" );
    foreach my $d ( qw/dlf ignored continuation errors/ ) {
        $self->assert( -d "$path/$d", "missing directory for category $d" );
        $self->assert( $store->has_category( $d ),
                       "category $d wasn't created" );
    }
    $self->assert( -f "$path/lock", "open() didn't create the lock file" );

    $store->close;
}

sub test_open {
    my $self = $_[0];

    my $path = $self->{workingdir} . "/test_open";
    $self->assert_died( sub { Lire::DlfStore->open( $path ) },
                        qr/DlfStore '$path' doesn't exist/ );

    my $store = Lire::DlfStore->open( $path, 1 );
    $self->assert_not_null( $store, "open() returned undef" );
    $store->close;
    $store = Lire::DlfStore->open( $path );
    $self->assert_not_null( $store, "open() returned undef" );
    $self->assert_equals( $path, $store->path );
    $store->close;
}

sub test_close {
    my $self = $_[0];

    my $path = $self->{workingdir} . "/test_close";
    my $store = Lire::DlfStore->open( $path, 1);
    $self->assert_not_null( $store, "open()) returned undef" );
    $store->close;
    $self->assert( ! -f "$path/lock", "close()) didn't remove the lock file" );
}

sub test_locked_open {
    my $self = $_[0];

    my $path = $self->{workingdir} . "/test_locked_open";
    my $store = Lire::DlfStore->open( $path, 1);
    $store->close;
    create_file( "$path/lock", getppid . "\n" );
    $self->assert_died( sub { Lire::DlfStore->open( $path ) },
                        qr/DlfStore '$path' is locked by process/ );
}

sub test_stale_lock {
    my $self = $_[0];

    my $path = $self->{workingdir} . "/test_state_lock";
    my $store = Lire::DlfStore->open( $path, 1);
    $self->assert_not_null( $store, "open()) returned undef" );
    $store->close;

    open LOCK, "> $path/lock"
      or die "can't create bogus lock file\n";
    print LOCK "-2\n";
    close LOCK;

    $store = Lire::DlfStore->open( $path );
    $self->assert_not_null( $store, "open() returned undef" );
    $store->close;
}

sub test_open_dlf_stream {
    my $self = $_[0];

    my $path = $self->{workingdir} . "/test_open_stream";
    my $store = Lire::DlfStore->open( $path, 1 );
    $self->assert_not_null( $store, "open() returned undef" );

    my @streams = $store->dlf_streams;
    $self->assert_equals( 0, scalar @streams );
    $self->assert( ! $store->has_dlf_stream( "test" ), 
                   "store shouldn't have a test DLF stream." );

    $self->assert_died( sub { $store->open_dlf_stream( "test", "r" ) },
                        qr/no DLF stream 'test' in this store/ );
    my $s = $store->open_dlf_stream( "test", "w" );
    $self->assert( $s->isa( "Lire::DlfStream"),
                   "open_dlf_stream didn't returned a Lire::DlfStream object");
    $self->assert_died( sub { $store->open_dlf_stream( "test", "r") },
                        qr/the DLF stream 'test' is already opened/  );
    $s->close;

    $s = $store->open_dlf_stream( "test", "r" );
    $self->assert( $s->isa( "Lire::DlfStream" ), "open_dlf_stream() failed" );

    $self->assert( $store->has_dlf_stream( "test"),
                   "has_dlf_stream() should have succeeded",
                 );
    @streams = $store->dlf_streams;
    $self->assert_deep_equals( [ "test"], \@streams );

    $s->close;
}

sub test_categories {
    my $self = $_[0];

    my $path = $self->{workingdir} . "/test_categories";
    my $store = Lire::DlfStore->open( $path, 1 );
    my @categories = qw/dlf continuation ignored errors /;
    my @cat = $store->categories;
    $store->close;
    $self->assert_deep_equals( [ sort @categories ],
                               [ sort @cat ] );
}

sub test_has_category {
    my $self = $_[0];

    my $path = $self->{workingdir} . "/test_has_category";
    my $store = Lire::DlfStore->open( $path, 1 );
    foreach my $c ( qw/dlf continuation ignored errors / ) {
        $self->assert( $store->has_category( $c ),
                       "has_category( '$c' ) should be true" );
    }
    $self->assert( ! $store->has_category( "no_such_cat" ),
                   "has_category( 'no_such_cat' ) should return false" );
    $store->close;
}

sub test_create_category {
    my $self = $_[0];

    my $path = $self->{workingdir} . "/test_create_category";
    my $store = Lire::DlfStore->open( $path, 1 );
    $store->create_category( "new_cat" );
    $self->assert( -d "$path/new_cat",
                   "directory $path/new_cat wasn't created" );
    $store->close;
}

sub test_storage_file {
    my $self = $_[0];

    my $path = $self->{workingdir} . "/test_storage_file";
    my $store = Lire::DlfStore->open( $path, 1 );
    $self->assert_died( sub { $store->storage_file },
                        qr/missing 'category' parameter/ );

    $self->assert_died( sub { $store->storage_file( 'dlf', "file with space")},
                        qr/invalid 'name' parameter:/ );

    $self->assert_died( sub { $store->storage_file( 'dlf' ) },
                        qr/missing 'name' parameter/ );

    $self->assert_died( sub { $store->storage_file( "test", "test" ) },
                        qr/category 'test' doesn't exist/ );

    $self->assert_equals( "$path/errors/test.log",
                          $store->storage_file( "errors", "test.log" ) );
    $store->close;
}

sub test_storage_files {
    my $self = $_[0];

    my $path = $self->{workingdir} . "/test_storage_file";
    my $store = Lire::DlfStore->open( $path, 1 );
    create_file( "$path/errors/test.log" );
    create_file( "$path/errors/test2.log" );

    $self->assert_died( sub { $store->storage_files },
                        qr/missing 'category' parameter/ );

    $self->assert_died( sub { $store->storage_files( "test" ) },
                        qr/category 'test' doesn't exist/ );

    my @e_files = sort qw/test.log test2.log/;
    my @files = sort $store->storage_files( "errors" );
    $self->assert_deep_equals( \@e_files, \@files );
    $store->close;
}
1;
