package tests::FunctionalTestCaseTest;

use strict;

use base qw/Lire::Test::TestCase/;

use Lire::Config::Parser;
use Lire::Config::Build qw/ac_info ac_path/;
use Lire::Test::FunctionalTestCase;
use Lire::Utils qw/tempfile tempdir create_file file_content/;
use File::Path;
use File::Basename;
use Cwd 'realpath';
use Symbol;
use Carp;

sub set_up  {
    my $self = $_[0];

    $self->set_up_fake_lire_installation();

    $self->{functional} = new Lire::Test::FunctionalTestCase;
    $self->{functional}->set_up;
}

sub set_up_fake_lire_installation {
    my $self = $_[0];

    $self->{lire_root} = tempdir( "lire_XXXXXX", CLEANUP => 1 );
    $self->{saved_build_infos} = \%Lire::Config::Build::ac;
    %Lire::Config::Build::ac =
      ( %{$self->{saved_build_infos}},
        bindir => "$self->{lire_root}/bin",
        sysconfdir => "$self->{lire_root}/etc",
        PACKAGE => 'lire',
        datadir => "$self->{lire_root}/share",
      );

    mkpath( [ "$self->{lire_root}/etc/lire",
              "$self->{lire_root}/bin",
              "$self->{lire_root}/libexec/lire",
              "$self->{lire_root}/share/lire/config-spec",
              "$self->{lire_root}/share/lire/schemas",
              "$self->{lire_root}/share/lire/filters",
              "$self->{lire_root}/share/lire/reports",
            ], 0, 0755 );

    create_file( "$self->{lire_root}/bin/lr_xml2report",
                 "#!/bin/sh -e\ntrue" );
    chmod 0755, "$self->{lire_root}/bin/lr_xml2report"
      or croak "chmod on lr_xml2report failed: $!\n";

    create_file( "$self->{lire_root}/etc/lire/defaults",
                 "#!/bin/sh -e\ntrue" );
    create_file( "$self->{lire_root}/share/lire/config-spec/lire.xml",
                 file_content( realpath( dirname( __FILE__)) . "/../config-spec/lire.xml" ));
}

sub tear_down {
    my $self = $_[0];

    %Lire::Config::Build::ac = %{$self->{saved_build_infos}};

    $self->{functional}->tear_down;
}

sub test_new {
    my $self = $_[0];

    my $test = new Lire::Test::FunctionalTestCase;
    $self->assert_not_null( $test, "new() returned undef" );
    $self->assert( UNIVERSAL::isa( $test, 'Lire::Test::FunctionalTestCase' ),
                   "new() returned wrong instance: $test" );

    chmod 0000, "$self->{lire_root}/bin/lr_xml2report"
      or croak "failed to remove permissions on lr_xml2report: $!";
    $self->assert_died( sub { new Lire::Test::FunctionalTestCase },
                        qr/isn't an executable. Was Lire installed?/);

    chmod 0755, "$self->{lire_root}/bin/lr_xml2report"
      or croak "failed to put back permissions on lr_xml2report: $!";

    chmod 0000, "$self->{lire_root}/etc/lire/defaults"
      or croak "failed to remove permissions on defaults: $!";
    $self->assert_died( sub { new Lire::Test::FunctionalTestCase },
                        qr/isn't present. Was Lire installed?/);
    chmod 0664, "$self->{lire_root}/etc/lire/defaults"
      or croak "failed to put back permission on defaults: $!";

    foreach my $d ( "$self->{lire_root}/etc/lire",
                    "$self->{lire_root}/share/lire" )
    {
        chmod 0555, $d
          or croak "failed to remove permissions on $d: $!";
        $self->assert_died( sub { new Lire::Test::FunctionalTestCase },
                            qr/you need write access to Lire's.*$d/ );
        chmod 0755, "$d"
          or croak "failed to remove permissions on $d: $!";
    }
}

sub test_set_up {
    my $self = $_[0];

    my $test = new Lire::Test::FunctionalTestCase;
    $test->set_up;
    foreach my $a ( qw/_rundir _stdout_file _stderr_file _homedir/ ) {
        $self->assert_not_null( $test->{$a}, "set_up() should set $a" );
    }

    foreach my $d ( qw/_rundir _homedir/ ) {
        $self->assert( -d $test->{$d}, "$d '$test->{$d}' wasn't created" );
    }

    foreach my $d ( qw/ etc reports schemas filters config converters / ) {
        my $dir = $test->{_homedir} . "/.lire/$d";
        $self->assert( -d $dir,  "directory '$dir' wasn't created" );
    }

    my ( $fh, $filename ) = tempfile( "testXXXXXX" );
    push @{$test->{_test_files}}, $filename;

    $test->tear_down;
    foreach my $d ( qw/_rundir _homedir/ ) {
        $self->assert( !-d $test->{$d}, "$d '$test->{$d}' wasn't deleted by tear_down" );
    }
    $self->assert( !-f $filename, "$filename should have been deleted" );
}

sub test_lire_run {
    my $self = $_[0];

    ;
    $self->assert_died( sub { $self->{functional}->lire_run },
                      qr/missing 'cmd' argument/);

    my $result = $self->{functional}->lire_run( "echo This is a test" );
    $self->assert_not_null( $result, "expected a result object" );
    $self->assert( UNIVERSAL::isa( $result, "Lire::Test::CommandResult" ),
                   "lire_run returned wrong object: $result" );

    $self->assert_equals( 0, $result->status );
    $self->assert_equals( "This is a test\n", $result->stdout );
    $self->assert( !$result->stderr, "stderr should be empty" );

    $result = $self->{functional}->lire_run( 'echo $PATH' );
    $self->assert_equals( 0, $result->status );

    my $bindir = ac_info( 'bindir' );
    $self->assert_matches( qr/^$bindir/, $result->stdout );
 }

sub test_create_test_file {
    my $self = $_[0];

    my $file = $self->{functional}->rundir . "/test_file";
    my $content = <<EOF;
This is the content of the file.

That was created.
EOF
    $self->{functional}->create_test_file( $file, $content );
    $self->assert( -f $file, "file $file doesn't exist" );
    $self->assert( grep { $_ eq $file } @{$self->{functional}{_test_files}},
                   "$file wasn't added to _test_files" );

    $self->assert_equals( $content, file_content( $file ) );

    ;
    $self->assert_died( sub { $self->{functional}->create_test_file( $file, $content ) },
                        qr/file '$file' already exists/ );

    ;
    $self->assert_died( sub { $self->{functional}->create_test_file },
                        qr/missing 'filename' argument/ );

    my $empty_file = $self->{functional}->rundir . "/empty_file";
    $self->{functional}->create_test_file( $empty_file );
    $self->assert( -f $empty_file, "file $empty_file wasn't created" );
    $self->assert( ! -s $empty_file, "$empty_file isn't empty" );
}

sub test_install_xml_spec {
    my $self = $_[0];

    my $fh;
    my $mock_file = $self->{functional}->rundir . "/test-mock.xml";
    open $fh, "> $mock_file"
      or croak "can't create $mock_file in rundir: $!";
    print $fh <<EOF;
Mock file
EOF
    close $fh;

    $self->assert_died( sub { $self->{functional}->install_xml_spec( "bad_param" ) },
                        qr/invalid spec_type parameter/ );

    my $base_site = ac_path( "datadir", "PACKAGE" );
    $self->{functional}->install_xml_spec( "site_filter", "func_test",
                                           $mock_file );
    $self->assert( -f $base_site . "/filters/func_test/test-mock.xml",
                   "file wasn't installed properly as site_filter" );
    $self->{functional}->install_xml_spec( "site_report", "func_test",
                                           $mock_file );
    $self->assert( -f $base_site . "/reports/func_test/test-mock.xml",
                   "file wasn't installed properly as site_report" );
    $self->{functional}->install_xml_spec( "site_schema", "test", $mock_file );
    $self->assert( -f $base_site . "/schemas/test-mock.xml",
                   "file wasn't installed properly as site_schema" );
    foreach my $f ( qw!filters/func_test/test-mock.xml
                       reports/func_test/test-mock.xml
                       schemas/test-mock.xml ! )
    {
        my $file = "$base_site/$f";
        my $found = grep { $_ eq $file } @{$self->{functional}{_test_files}};
        $self->assert( $found, "file $file wasn't added to list of files to remove" );
    }

    my $base_home = $self->{functional}->homedir . "/.lire";
    $self->{functional}->install_xml_spec( "filter", "func_test", $mock_file );
    $self->assert( -f $base_home . "/filters/func_test/test-mock.xml",
                   "file wasn't installed properly as filter" );
    $self->{functional}->install_xml_spec( "report", "func_test", $mock_file );
    $self->assert( -f $base_home . "/reports/func_test/test-mock.xml",
                   "file wasn't installed properly as report" );
    $self->{functional}->install_xml_spec( "schema", "tes", $mock_file );
    $self->assert( -f $base_home . "/schemas/test-mock.xml",
                   "file wasn't installed properly as schema" );

    foreach my $f ( qw!filters/func_test/test-mock.xml
                       reports/func_test/test-mock.xml
                       schemas/test-mock.xml ! )
    {
        my $file = "$base_home/$f";
        my $found = grep { $_ eq $file } @{$self->{functional}{_test_files}};
        $self->assert( $found, "file $file wasn't added to list of files to remove" );
    }
}

sub test_config_spec {
    my $self = $_[0];

    my $spec = $self->{functional}->config_spec;
    $self->assert_not_null( $spec, "config_spec() returned undef" );
    $self->assert( UNIVERSAL::isa( $spec, 'Lire::Config::Spec::Config' ),
                   "config_spec() didn't return a Lire::Config::Spec::Config: $spec" );
}

sub test_setup_sendmail_path {
    my $self = $_[0];

    my $config_file = $self->{functional}->homedir . "/.lire/config/func_test_sendmail.xml";
    $self->assert_not_null( $self->{functional}{_config_files}{func_test_sendmail},
                            "no config file 'func_test_sendmail' was created"
                          );

    my $conf = $self->{functional}{_config_files}{func_test_sendmail};
    my $e_sendmail = $self->{functional}->rundir . "/func_test_sendmail";
    my $sendmail_path = $conf->global->get( 'sendmail_path' )->as_value;
    $self->assert_not_null( $sendmail_path, "sendmail_path wasn't set" );
    $self->assert_equals( $e_sendmail, $sendmail_path );
    $self->assert( -x $sendmail_path, "$sendmail_path isn't executable" );

    # Make sure only one var was set
    foreach my $var ( $conf->global->spec->components() ) {
        next if $var->name eq 'sendmail_path';
        $self->assert( ! $conf->global->is_set( $var->name ),
                       $var->name . " was set in $config_file" );
    }

    my $maildir = $self->{functional}->rundir . "/.func_test_mail";
    $self->assert( -d $maildir, "directory to store sent mail doesn't exists: $maildir" );
}

sub test_sent_mail {
    my $self = $_[0];

    my $content = <<EOM;
From: me
To: you
Subject: Test mail

Test
EOM
    my $fh = gensym;
    my $sendmail_path = $self->{functional}->rundir . "/func_test_sendmail";
    open $fh, "| $sendmail_path test1 test.2 test2"
      or croak "open $sendmail_path failed: $!";
    print $fh $content;
    close $fh
      or croak "close failed: $!";

    open $fh, "| $sendmail_path recipient1 recipient2"
      or croak "open $sendmail_path failed: $!";
    print $fh $content;
    close $fh
      or croak "close failed: $!";

    my $result = $self->{functional}->lire_run( "true" );
    my $msg = $result->sent_mail;
    $self->assert_equals( 2, scalar @$msg );
    for my $idx ( 0..1 ) {
        $self->assert_not_null( $msg->[$idx], "Message $idx structure is undef");
        $self->assert_not_null( $msg->[$idx]{recipients},
                                "Message $idx 'recipients' is undef");
        $self->assert_not_null( $msg->[$idx]{message},
                                "Message $idx 'message' is undef");
        $self->assert_equals( 2, scalar (keys %{$msg->[$idx]}) );
    }
    $self->assert_deep_equals( [ qw/test1 test.2 test2/ ],
                               $msg->[0]{recipients} );
    $self->assert_deep_equals( [ qw/recipient1 recipient2/ ],
                               $msg->[1]{recipients} );
    $self->assert_equals( $content, $msg->[0]{message} );
    $self->assert_equals( $content, $msg->[1]{message} );

    $result = $self->{functional}->lire_run( "true" );
    $self->assert_equals( 0, scalar @{$result->sent_mail} );
}

sub test_create_cfg_file {
    my $self = $_[0];

    $self->assert_died( sub { $self->{functional}->create_test_cfg_file() },
                        qr/missing 'name' parameter/ );

    $self->assert_died( sub { $self->{functional}->create_test_cfg_file( "../test" ) },
                        qr/'name' parameter should only contains digits, letters/ );

    my $cfg = $self->{functional}->create_test_cfg_file( "test" );
    $self->assert_not_null( $cfg, "create_test_cfg_file() returned undef" );
    $self->assert( UNIVERSAL::isa( $cfg, 'Lire::Config::ConfigFile' ),
                   "create_test_cfg_file() should return a Lire::Config::ConfigFile: $cfg"
                 );
    my $cfg_file = $self->{functional}->homedir . "/.lire/config/test.xml";
    $self->assert_equals( $cfg_file, $cfg->filename );
    $self->assert( ! -f $cfg_file, "$cfg_file shouldn't exist" );
    $self->assert_not_null( $cfg->global, "config's global section wasn't initialzied" );
    $self->assert_equals($cfg->global->spec, $self->{functional}->config_spec);

    $self->assert_equals( $cfg, $self->{functional}->create_test_cfg_file( "test" ));

    my $result = $self->{functional}->lire_run( "echo Hi" );
    $self->assert( -f $cfg_file, "$cfg_file wasn't created" );
}

1;
