package tests::ReportSpecTest;

use strict;

use base qw/ Lire::Test::TestCase /;

use Lire::ReportSpec;
use Lire::DlfSchema;
use Lire::FilterExpr;
use Lire::Param;
use Lire::Group;
use Lire::GroupField;
use Lire::GroupOp;

use File::Temp qw/mktemp tempdir/;
use File::Basename qw/dirname/;
use Cwd qw/realpath/;
use Carp;
use Symbol;

sub new {
    my $self = shift->SUPER::new( @_ );

    $self->{tmpdir} = tempdir( "ReportSpecTest_XXXXXX", CLEANUP => 1 );
    $self->{tmp_test_spec_dir} = $self->{tmpdir} . "/test";
    mkdir $self->{tmp_test_spec_dir}
      or croak "failed to make $self->{tmp_test_spec_dir}: $!\n";
    $self->{dir}    = realpath( dirname(__FILE__) );

    $self;
}

sub set_up {
    my $self = shift->SUPER::set_up();

    $self->{cfg}{lr_reports_path} = [ $self->{dir} . "/reports",
                                      $self->{tmpdir} ];;
    $self->{cfg}{lr_schemas_path} = [ $self->{dir} . "/schemas" ];
}

sub test_non_existent_report {
    my $self = $_[0];

    my $spec = eval { Lire::ReportSpec->load( "test", "non-existent" ) };
    $self->assert_not_null( $@, "load() should have thrown an exception" );
}

sub test_load_report {
    my $self = $_[0];

    my $spec = Lire::ReportSpec->load( "test", "top-files" );
    $self->assert_equals( "top-files", $spec->id );
}

sub create_test_report_spec {
    my $self = $_[0];

    my $spec = new Lire::ReportSpec();
    $spec->id( "test-report" );
    $spec->superservice( "test" );
    $spec->title( "Test Report Specification" );
    $spec->description( "<para>Test Report Specification</para>" );
    $spec->display_title( "Test Report" );
    $spec->param( "param_int", new Lire::Param( name => "param_int",
                                                type => "int" ) );
    $spec->param( "param_int" )->default( 10 );
    $spec->param( "param_match", new Lire::Param( name => "param_match",
                                                  type => "string" ) );
    $spec->param( "param_match" )->default( ".*" );

    $spec->filter_spec( new Lire::FilterExpr::Match( container => $spec,
                                                     value => '$file',
                                                     re => '$param_match',
                                                   )
                      );

    my $group = new Lire::Group( report_spec => $spec,
                                limit => '$param_int', );

    $group->group_fields( [ new Lire::GroupField( report_spec => $spec,
                                                  name => "file",
                                                ) ] );
    $group->ops( [ new Lire::GroupOp::Count( report_spec => $spec,
                                             parent => $group,
                                             name => "downloads",
                                             label => "Downloads",
                                           ) ] );
    $group->sort_fields( [ "-downloads" ] );
    $spec->calc_spec( $group );

    $spec;
}

sub test_print {
    my $self = $_[0];

    my $spec = $self->create_test_report_spec;

    my $fh = gensym;
    my $specfile = "$self->{tmp_test_spec_dir}/test-report.xml";
    open $fh, "> $specfile"
      or croak "failed to open $specfile for writing: $!\n";
    $spec->print( $fh );
    close $fh
      or croak "close failed: $!\n";

    my $spec2 = Lire::ReportSpec->load( $spec->superservice, $spec->id );

    # The specs are going to have different keys which isn't an
    # error
    $spec2->{key} = $spec->{key};

    $self->assert_deep_equals( $spec, $spec2 );
}

1;



