package tests::UtilsTest;

use strict;

use base qw/ Lire::Test::TestCase /;

use File::Temp qw/ :mktemp /;
use File::Spec;
use Lire::Utils qw/xml_encode diff_lists tilde_expand
                   read_keyvalue_file write_keyvalue_file
                   tmpdir tempfile tempdir min max ratio ratio100
                   shell_quote file_content create_file
                  /;
use Carp;

sub set_up {
    my $self = $_[0];

    $self->{file} = mktemp( "keyvalueXXXXXX");
}

sub tear_down {
    my $self = $_[0];

    unlink $self->{file};
}

sub test_xml_encode {
    my $self = $_[0];

    my $v = xml_encode( "<XML> & </XML> Joe \"Such\" a Friend's birtday" );
    $self->assert_equals( "&lt;XML> &amp; &lt;/XML> Joe &quot;Such&quot; a Friend&apos;s birtday", $v );
}

sub test_diff_lists {
    my $self = $_[0];

    my $diff = diff_lists( [qw/field1 field2 field3 field4/],
                           [qw/field4 field3 field2 field1/] );
    $self->assert_null( $diff, "should have found no diffs" );

    $diff = diff_lists( [qw/field1 field2 field3 field4/],
                        [qw/field3 field2 field4 field5/],
                      );
    $self->assert_not_null( $diff, "should have found diffs" );
    $self->assert_deep_equals( [ "field1" ], $diff->{remove} );
    $self->assert_deep_equals( [ "field5" ], $diff->{new} );

    $diff = diff_lists( [qw/field1 field2 field3 field4/],
                        [qw/field6 field3 field2 field4 field5/],
                      );

    $self->assert_not_null( $diff, "should have found diffs" );
    $self->assert_deep_equals( [ "field1" ], $diff->{remove} );
    $self->assert_deep_equals( [ "field5", "field6" ], $diff->{new} );
}

sub test_keyvalue_file {
    my $self = $_[0];

    my %dict = (
                Schema => "www",
                Fields => "field1 field2 field3 field4",
               );

    write_keyvalue_file( $self->{file}, \%dict );
    $self->assert( -f $self->{file}, "File $self->{file} wasn't created" );
    my $r = read_keyvalue_file( $self->{file} );
    $self->assert_deep_equals( \%dict, $r );
    unlink $self->{file};

}

sub test_keyvalue_file_with_nulls {
    my $self = $_[0];

    my %dict = (
                Records => 0,
                'Start-Time' => undef,
                'End-Time' => undef,
               );

    write_keyvalue_file( $self->{file}, \%dict );
    $self->assert( -f $self->{file}, "File $self->{file} wasn't created" );
    my $r = read_keyvalue_file( $self->{file} );
    $self->assert_deep_equals( \%dict, $r );
    unlink $self->{file};
}

sub test_tilde_expansion {
    my $self = $_[0];

    my $home = $ENV{'HOME'};

    $self->assert_equals($home, tilde_expand('~'));
    $self->assert_equals("$home/", tilde_expand('~/'));
    $self->assert_equals("$home/~", tilde_expand('~/~'));
    $self->assert_equals("$home/~/", tilde_expand('~/~/'));
    $self->assert_equals('/~', tilde_expand('/~'));
    $self->assert_equals('/~/', tilde_expand('/~/'));

    my $user = 'uioxzcyo';

    $self->assert_equals("~$user", tilde_expand("~$user"));
    $self->assert_equals("~$user/", tilde_expand("~$user/"));
    $self->assert_equals("/$user~", tilde_expand("/$user~"));
    $self->assert_equals("/$user~/", tilde_expand("/$user~/"));

    $user = getpwuid($<);
    my ($name,$pass,$uid,$gid,$quota,$comment,$gecos,$dir) = getpwnam($user)
        or return;

    $self->assert_equals("$dir", tilde_expand("~$user"));
    $self->assert_equals("$dir/", tilde_expand("~$user/"));
    $self->assert_equals("/$user~", tilde_expand("/$user~"));
    $self->assert_equals("/$user~/", tilde_expand("/$user~/"));
}

sub test_minmax {
	my $self = $_[0];

    $self->assert_null(max());

    $self->assert_equals(-1, max(-1));
    $self->assert_equals(0, max(0));
    $self->assert_equals(1, max(1));

    $self->assert_equals(-1, max(-1,-2));
    $self->assert_equals(0, max(0,-2));
    $self->assert_equals(1, max(1,-2));
    $self->assert_equals(-1, max(-2,-1));
    $self->assert_equals(0, max(-2,0));
    $self->assert_equals(1, max(-2,1));
    $self->assert_equals(-1, max(-1,-1));
    $self->assert_equals(0, max(0,0));
    $self->assert_equals(1, max(1,1));
    $self->assert_equals(2, max(-1,2));
    $self->assert_equals(2, max(0,2));
    $self->assert_equals(2, max(1,2));
    $self->assert_equals(2, max(-3,2));
    $self->assert_equals(3, max(3,2));

    $self->assert_equals(0, max(0,0,0));
    $self->assert_equals(1, max(1,0,0));
    $self->assert_equals(1, max(0,1,0));
    $self->assert_equals(1, max(0,0,1));
    $self->assert_equals(0, max(-1,0,0));
    $self->assert_equals(0, max(0,-1,0));
    $self->assert_equals(0, max(0,0,-1));

    $self->assert_null(min());

    $self->assert_equals(-1, min(-1));
    $self->assert_equals(0, min(0));
    $self->assert_equals(1, min(1));

    $self->assert_equals(-2, min(-1,-2));
    $self->assert_equals(-2, min(0,-2));
    $self->assert_equals(-2, min(1,-2));
    $self->assert_equals(-2, min(-2,-1));
    $self->assert_equals(-2, min(-2,0));
    $self->assert_equals(-2, min(-2,1));
    $self->assert_equals(-1, min(-1,-1));
    $self->assert_equals(0, min(0,0));
    $self->assert_equals(1, min(1,1));
    $self->assert_equals(-1, min(-1,2));
    $self->assert_equals(0, min(0,2));
    $self->assert_equals(1, min(1,2));
    $self->assert_equals(-3, min(-3,2));
    $self->assert_equals(2, min(3,2));

    $self->assert_equals(0, min(0,0,0));
    $self->assert_equals(0, min(1,0,0));
    $self->assert_equals(0, min(0,1,0));
    $self->assert_equals(0, min(0,0,1));
    $self->assert_equals(-1, min(-1,0,0));
    $self->assert_equals(-1, min(0,-1,0));
    $self->assert_equals(-1, min(0,0,-1));
}

sub test_tmpdir {
    my $self = $_[0];

    $self->assert_equals( File::Spec->tmpdir, tmpdir() );
    {
        local $ENV{TMPDIR} = $ENV{HOME};
        $self->assert_equals( $ENV{HOME}, Lire::Utils::_tmpdir() );
    }
}

sub test_tempfile {
    my $self = $_[0];

    my $fh = Lire::Utils::_tempfile();
    $self->assert_not_null( $fh, "_tempfile() returned undef" );
    $self->assert( -r $fh, "file handle isn't readable" );
    $self->assert( -w $fh, "file handle isn't writeable" );

    my ( $nfh, $filename ) = Lire::Utils::_tempfile( "lireXXXXXX" );
    $self->assert( qr/lire[a-zA-Z0-9]{6}$/, $filename );
    $self->assert( -r $nfh, "file handle isn't readable" );
    $self->assert( -w $nfh, "file handle isn't writeable" );
    unlink $filename;

    # Try with perl's standard ones. (PerlUnit only works with perl 5.6.1 and
    # later
    my $pfh = tempfile();
    $self->assert_not_null( $pfh, "tempfile() returned undef" );
    $self->assert( -r $pfh, "file handle isn't readable" );
    $self->assert( -w $pfh, "file handle isn't writeable" );

    my ( $pnfh, $pfilename ) = tempfile( "lireXXXXXX" );
    $self->assert( qr/lire[a-zA-Z0-9]{6}$/, $pfilename );
    $self->assert( -r $pnfh, "file handle isn't readable" );
    $self->assert( -w $pnfh, "file handle isn't writeable" );
    unlink $pfilename;
}

sub test_tempdir {
    my $self = $_[0];

    my $tmpdir = Lire::Utils::_tempdir();
    $self->assert_not_null( $tmpdir, "_tempdir() returned undef" );
    $self->assert( -r $tmpdir, "tempory dir isn't readable" );
    $self->assert( -w $tmpdir, "tempory dir isn't writeable" );
    $self->assert( -x $tmpdir, "tempory dir isn't searchable" );
    rmdir $tmpdir;

    $tmpdir = Lire::Utils::_tempdir( "lireXXXXXX" );;
    $self->assert_not_null( $tmpdir, "_tempdir() returned undef" );
    $self->assert( qr/lire[a-zA-Z0-9]{6}$/, $tmpdir );
    $self->assert( -r $tmpdir, "tempory dir isn't readable" );
    $self->assert( -w $tmpdir, "tempory dir isn't writeable" );
    $self->assert( -x $tmpdir, "tempory dir isn't searchable" );
    rmdir $tmpdir;

    # Try with perl's standard ones. (PerlUnit only works with perl 5.6.1 and
    # later
    my $ptmpdir = tempdir();
    $self->assert_not_null( $ptmpdir, "tempdir() returned undef" );
    $self->assert( -r $ptmpdir, "tempory dir isn't readable" );
    $self->assert( -w $ptmpdir, "tempory dir isn't writeable" );
    $self->assert( -x $ptmpdir, "tempory dir isn't searchable" );
    rmdir $ptmpdir;

    $ptmpdir = tempdir( "lireXXXXXX" );;
    $self->assert_not_null( $ptmpdir, "_tmpdir() returned undef" );
    $self->assert( qr/lire[a-zA-Z0-9]{6}$/, $ptmpdir );
    $self->assert( -r $ptmpdir, "tempory dir isn't readable" );
    $self->assert( -w $ptmpdir, "tempory dir isn't writeable" );
    $self->assert( -x $ptmpdir, "tempory dir isn't searchable" );
    rmdir $ptmpdir;
}

sub test_ratio {
    my $self = $_[0];

    $self->assert_equals( 2, ratio( 10, 5 ) );
    # Force string comparison since NaN == NaN is false
    $self->assert_str_equals( "NaN", ratio( 10, 0 ) );
    $self->assert_equals( 0, ratio( 0, 10 ) );
    $self->assert_equals( "3.33", ratio( 10, 3 ) );
}

sub test_ratio100 {
    my $self = $_[0];

    $self->assert_equals( 50, ratio100( 5, 10 ) );
    # Force string comparison since NaN == NaN is false
    $self->assert_str_equals( "NaN", ratio100( 0, 0 ) );
    $self->assert_equals( 0, ratio100( 0, 10 ) );
    $self->assert_equals( "33.3", ratio100( 3333, 10000 ) );
}

sub test_shell_quote {
    my $self = $_[0];

    $self->assert_equals( "''", shell_quote( undef ) );
    $self->assert_equals( "'Test'", shell_quote( 'Test' ) );
    $self->assert_equals( q{'Test'\'''}, shell_quote( "Test'" ) );
    $self->assert_equals( q{'`echo shell'\''s test`'}, shell_quote( "`echo shell's test`" ) );
}

sub test_file_content {
    my $self = $_[0];

    my $content = <<EOF;
This is the file content.

More than one line.
EOF
    my ($fh, $name ) = File::Temp::tempfile();

    print $fh $content;
    close $fh;

    $self->assert_died( sub { file_content() },
                        qr/missing 'filename' parameter/ );
    $self->assert_died( sub { file_content( "/no/such/file" ) },
                        qr/No such file/ );

    $self->assert_equals( $content, file_content( $name ) );
    unlink $name;
}

sub test_create_file {
    my $self = $_[0];

    $self->assert_died( sub { create_file()}, 
                        qr/missing 'filename' parameter/ );

    $self->assert_died( sub { create_file( "/no/such/file" ) },
                        qr/error creating.*: No such file/ );

    my $content = <<EOF;
This is a file.

And its content.
EOF

    my $tmpdir = File::Temp::tempdir( "Utils_XXXXXX", CLEANUP => 1 );
    my $filename = "$tmpdir/test_file";
    create_file( $filename );
    $self->assert( -f $filename, "$filename wasn't created" );
    $self->assert( ! -s $filename, "$filename should be empty" );

    create_file( $filename, $content );

    my $fh;
    open $fh, $filename
      or croak "open failed: $!";
    local $/;
    my $new_content = <$fh>;
    close $fh;

    $self->assert_equals( $content, $new_content );
}

1;
