package tests::functional::DefaultsTest;

use strict;

use base qw/Lire::Test::FunctionalTestCase/;

use File::Path;
use File::Basename;
use Lire::Config::Build qw/ac_info ac_path/;
use Lire::Utils qw/tempdir create_file/;
use Carp;

sub new {
    my $self = shift->SUPER::new( @_ );

    $self->{defaults_file} = ac_info( "sysconfdir") . "/lire/defaults";

    return $self;
}

sub test_tmpdir_unset {
    my $self = $_[0];

    local $ENV{TMPDIR} = '';

    my $result = $self->lire_run( ". $self->{defaults_file}; env" );
    $self->annotate( $result->stderr );
    $self->assert_equals( 0, $result->status );
    $self->assert_matches( qr!TMPDIR=/tmp!, $result->stdout );
}

sub test_tmpdir_set {
    my $self = $_[0];

    my $tmpdir = $self->homedir . "/tmp";
    local $ENV{TMPDIR} = $tmpdir;

    my $result = $self->lire_run( ". $self->{defaults_file}; env" );
    $self->annotate( $result->stderr );
    $self->assert_equals( 0, $result->status );
    $self->assert_matches( qr!TMPDIR=/tmp!, $result->stdout );

    mkdir $tmpdir,0700
      or croak "mkdir $tmpdir failed $!";
    $result = $self->lire_run( ". $self->{defaults_file}; env" );
    $self->annotate( $result->stderr );
    $self->assert_equals( 0, $result->status );
    $self->assert_matches( qr!TMPDIR=$tmpdir!, $result->stdout );
}

sub test_lr_environment_run {
    my $self = $_[0];
    my $result = $self->lire_run( ". $self->{defaults_file}; env" );
    $self->annotate( $result->stderr );
    $self->assert_equals( 0, $result->status );
    $self->assert_matches( qr/lr_environment info started with/,
                           $result->stderr );
}

sub test_config_migration_detection {
    my $self = $_[0];

    my $sysconfdir = ac_path( 'sysconfdir', 'PACKAGE' );
    my $etchomedir = $self->homedir . "/.lire/etc";

    my @files = split ' ', "$sysconfdir/defaults.local
                       $sysconfdir/ext_defaults.local
                       $etchomedir/defaults $etchomedir/ext_defaults";
    foreach my $f ( @files )
    {
        $self->create_test_file( $f, <<EOF );
# Dummy config to test migration
EOF
    }
    my $result = $self->lire_run( ". $self->{defaults_file}" );
    $self->annotate( $result->stderr );
    $self->assert_not_equals( 0, $result->status );

    foreach my $f ( @files ) {
        my $new_file = $self->xml_file_from_old_file( $f );
        $self->assert_matches( qr/run lr_env2conf $f/, $result->stderr );
    }
}

sub test_obsolete_config_migration_detection {
    my $self = $_[0];

    my $sysconfdir = ac_path( 'sysconfdir', 'PACKAGE' );
    my $etchomedir = $self->homedir . "/.lire/etc";

    my @files = split ' ', "$sysconfdir/defaults.local
                       $sysconfdir/ext_defaults.local
                       $etchomedir/defaults $etchomedir/ext_defaults";
    foreach my $f ( @files )
    {
        # XML files with a mtime < shell script should also trigger
        # a migration request.
        $self->create_test_file( $self->xml_file_from_old_file($f), <<EOF );
<?xml version="1.0" encoding="ISO-8859-1"?>
<config xmlns="http://www.logreport.org/LRCML/">
 <global/>
</config>
EOF

        $self->create_test_file( $f, <<EOF );
# Dummy config to test migration
EOF
    }
    my $result = $self->lire_run( ". $self->{defaults_file}" );
    $self->annotate( $result->stderr );
    $self->assert_not_equals( 0, $result->status );

    foreach my $f ( @files ) {
        my $new_file = $self->xml_file_from_old_file( $f );
        $self->assert_matches( qr/run lr_env2conf $f/, $result->stderr );
    }
}

sub xml_file_from_old_file {
    my ( $self, $old_file ) = @_;

    my ($name, $path ) = fileparse( $old_file );
    if ( index( $path, $self->homedir ) == 0 ) {
        return $self->homedir . "/.lire/config/$name.xml";
    } else {
        return ac_path( 'sysconfdir', 'PACKAGE' ) . "/config/$name.xml";
    }
}

sub test_old_config_files_loaded {
    my $self = $_[0];

    my $sysconfdir = ac_info( 'sysconfdir' ) . "/lire";
    my $etchomedir = $self->homedir . "/.lire/etc";

    my %files = map { my ($var) = m,([^/]+)$,; $var =~ tr/a-z./A-Z_/;
                      $_ => $var }
       split( /\s+/, "$sysconfdir/extension_defaults $sysconfdir/defaults.local
           $sysconfdir/ext_defaults.local
           $etchomedir/defaults $etchomedir/ext_defaults");
    while (my ( $file, $var ) = each %files ) {
        $self->create_test_file( $file, <<EOF );
$var=1
export $var
EOF
        my $xml_file = $self->xml_file_from_old_file($file);
        $self->create_test_file( $xml_file, <<EOF );
<?xml version="1.0" encoding="ISO-8859-1"?>
<config xmlns="http://www.logreport.org/LRCML/">
 <global/>
</config>
EOF
        my $newer= (stat($file))[9] + 5;
        utime ($newer, $newer, $xml_file) == 1
          or croak "utime on $xml_file failed: $!";
    }

    my $result = $self->lire_run( ". $self->{defaults_file}; env" );
    $self->annotate( $result->stderr );
    $self->assert_equals( 0, $result->status );

    for my $var ( values %files) {
        $self->assert_matches( qr/$var=1/, $result->stdout );
    }
}

sub test_home_directory_setup {
    my $self = $_[0];

    # Delete the .lire directory created by our parent
    # We create in the cmd to lire_run() because the $HOME/.lire/config
    # needs to be created when lire_run() is called.
    my $homedir = $self->homedir . "/.lire";
    my $result = $self->lire_run( "rm -fr $homedir; . $self->{defaults_file}; env" );
    $self->annotate( $result->stderr );
    $self->assert_equals( 0, $result->status );

    foreach my $d ( qw/config converters etc reports schemas filters/ ) {
        $self->assert( -d $self->homedir . "/.lire/$d",
                       "home directory '.lire/$d' wasn't created"
                     );
        $self->assert_matches( qr/creating.*$d directory/, $result->stderr );
    }
}

sub test_lr_tempfile {
    my $self = $_[0];

    my $result = $self->lire_run( <<EOS );
. $self->{defaults_file}
lr_tempfile test_XXXXXX.log
EOS
    $self->annotate( $result->stderr );
    $self->assert_equals( 0, $result->status );
    $self->assert_does_not_match( qr/ (crit|err|warning) / , $result->stderr );
    my $file = $result->stdout;
    chomp $file;
    $self->assert_matches( qr/test_[a-zA-Z0-9]{6}\.log$/, $file );
    $self->assert( -f $file, "file '$file' doesn't exists" );
}

sub test_lr_tempdir {
    my $self = $_[0];

    my $result = $self->lire_run( <<EOS );
. $self->{defaults_file}
lr_tempdir test_XXXXXX
EOS
    $self->annotate( $result->stderr );
    $self->assert_equals( 0, $result->status );
    $self->assert_does_not_match( qr/ (crit|err|warning) / , $result->stderr );
    my $dir = $result->stdout;
    chomp $dir;
    $self->assert_matches( qr/test_[a-zA-Z0-9]{6}/, $dir );
    $self->assert( -d $dir, "directory '$dir' doesn't exists" );
}

sub test_cleanup {
    my $self = $_[0];

    my $file = $self->rundir . "/test_file";
    create_file( $file, "This is a test\n" );
    my $dir = tempdir( "test_XXXXXX" );
    my $result = $self->lire_run( <<EOS );
. $self->{defaults_file}
lr_mark_for_cleanup $dir
lr_mark_for_cleanup $file
EOS
    $self->annotate( $result->stderr );
    $self->assert_does_not_match( qr/ (crit|err|warning) /, $result->stderr );
    $self->assert( !$result->stdout, "STDOUT should be empty" );
    $self->assert( ! -f $file, "$file wasn't removed" );
    $self->assert( ! -d $file, "$dir wasn't removed" );
}

sub test_keep_cleanup {
    my $self = $_[0];

    my $file = $self->rundir . "/test_file";
    $self->create_test_file( $file, "This is a test\n" );
    my $dir = tempdir( "test_XXXXXX" );
    my $result = $self->lire_run( <<EOS );
. $self->{defaults_file}
lr_keep=1
lr_mark_for_cleanup $dir $file
EOS
    $self->annotate( $result->stderr );
    $self->assert_does_not_match( qr/ (crit|err|warning) /, $result->stderr );
    $self->assert_matches( qr/ debug lr_keep is set, not deleting '$file'/,
                           $result->stderr );
    $self->assert_matches( qr/ debug lr_keep is set, not deleting '$dir'/,
                           $result->stderr );
    $self->assert( !$result->stdout, "STDOUT should be empty" );
    $self->assert( -f $file, "$file was removed even if with lr_keep=1" );
    $self->assert( -d $dir, "$dir was removed even if with lr_keep=1" );
}

1;
